@@ -90,7 +90,10 @@ test_that("range with NA", {
 test_that("long range", {
   skip_on_cran()
   skip_on_travis()
-  x <- integer(.Machine$integer.max + 5)
+  x <- tryCatch(integer(.Machine$integer.max + 5),
+                error = function(e) {
+                  skip("vector specified too large.")
+                })
   R <- range_rcpp(x)
   expect_equal(R[1], 0)
   expect_equal(R[2], 0)
